#include "Iface.hxx"
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <errno.h>

Iface::Iface()
{
	loaded = false;
}

Iface::Iface (const string ifaceDir, const string name)
{
	this->ifaceDir = ifaceDir;
	this->name = name;
	loaded = pickup_options();
}

string Iface::name2type (string name)
{
	size_t i, s = name.size();
	for (i = 0; i < s; i++)
		if (!isalpha (name[i]))
			break;
	string strip (name, 0, i);
	return strip;
}

bool Iface::pickup_options()
{
	if (ifaceDir == "") return false;
	VarInfoMap varInfoMap;
	VarInfoMap::loadDefaults (varInfoMap);
	options.load (ifaceDir + "/" + name + "/options", varInfoMap);
	string iftype;
	if (!options.exists ("TYPE"))
		iftype = name2type (name);
	else
		iftype = options["TYPE"];
	if (iftype == "lo" || iftype == "dummy")
		opt_type = IFTYPE_VIRTUAL;
	else
	if (iftype == "eth" || iftype == "plip" || iftype == "usb" || iftype == "dvb")
		opt_type = IFTYPE_REALPHYS;
	else
	if (iftype == "vlan" || iftype == "bond" || iftype == "bri" || iftype == "teql")
		opt_type = IFTYPE_HOSTEDPHYS;
	else
	if (iftype == "iptun" || iftype == "ipsectun")
		opt_type = IFTYPE_INDEPLOG;
	else
	if (iftype == "ppp")
		opt_type = IFTYPE_DEPLOG;
	else
		return false;
	default_options.load (ifaceDir + "/default/options", varInfoMap);
	type_options.load (ifaceDir + "/default/options-" + iftype, varInfoMap);
	pickup_qos_tree();
	return true;
}

bool Iface::pickup_qos_tree()
{
	if (ifaceDir == "") return false;
	string qosDir = ifaceDir + "/" + name + "/qos";
	// find first dir
	DIR * qosdir = opendir (qosDir.c_str());
	if (!qosdir)
	{
		if (errno != ENOENT)
			perror ("Iface::pickup_qos_tree()");
		return false;
	}
	struct dirent * dirEntry = NULL;
	while ((dirEntry = readdir (qosdir)))
	{
		struct stat statbuf;
		if (lstat ((qosDir + "/" + dirEntry->d_name).c_str(), &statbuf))
		{
			perror ("lstat");
			continue;
		}
		if (!S_ISDIR (statbuf.st_mode))
			continue;
		if (dirEntry->d_name[0] == '.')
			continue;
		if (string (dirEntry->d_name) == "CVS")
			continue;
		// start building root object
		qosRoot.setDev (name);
		return qosRoot.buildFromDir (qosDir + "/" + dirEntry->d_name, dirEntry->d_name, "root");
		// and exit
		break;
	}
	closedir (qosdir);
	return false;
}

void Iface::dump()
{
	printf ("dumping data for iface %s\n", name.c_str());
	printf ("default options:\n");
	default_options.dump();
	printf ("type options:\n");
	type_options.dump();
	printf ("iface options:\n");
	options.dump();
}

bool Iface::isLoaded()
{
	return loaded;
}

string Iface::getName()
{
	return name;
}
