#include "IfaceView.hxx"
#include "VarMap.hxx"
#include <map>

// see VarInfo.hxx:VarType
string vartype2str[] =
{
	"string",
	"integer",
	"bool",
	"enum"
};

IfaceView::IfaceView() :
ifoptVBox (Gtk::BUTTONBOX_START),
typeoptVBox (Gtk::BUTTONBOX_START),
defoptVBox (Gtk::BUTTONBOX_START),
buttonOk (Gtk::Stock::OK),
buttonCancel (Gtk::Stock::CANCEL),
ifoptFrame ("interface options"),
typeoptFrame ("type options"),
defoptFrame ("default options"),
qosFrame ("QoS")
{
	set_homogeneous (false);
	pack_start (ifoptFrame);
	pack_start (typeoptFrame);
	pack_start (defoptFrame);
	pack_start (qosFrame);
	ifoptFrame.add (ifoptNB);
	typeoptFrame.add (typeoptNB);
	defoptFrame.add (defoptNB);
	ifoptFrame.set_sensitive (false);
	typeoptFrame.set_sensitive (false);
	defoptFrame.set_sensitive (false);
	ifoptNB.append_page (ifoptVBox, "default profile");
	typeoptNB.append_page (typeoptVBox, "default profile");
	defoptNB.append_page (defoptVBox, "default profile");
	qosFrame.add (qosView);
	pack_start (hBox);
	hBox.pack_start (buttonOk);
	hBox.pack_start (buttonCancel);
	buttonOk.set_sensitive (false);
	buttonCancel.set_sensitive (false);
	show_all_children();
}

void IfaceView::set (Iface i)
{
	iface = i;
	rebuild (i.getName());
}

void IfaceView::set (const string s)
{
	rebuild (s);
}

void IfaceView::rebuild (string text)
{
	// FIXME: removing std:: breaks compilation :-/
	ifoptVBox.children().clear();
	for (std::map <string, VarInstance>::const_iterator iter = iface.options.begin(); iter != iface.options.end(); iter++)
	{
		string optname = (*iter).first;
		VarInstance inst = (*iter).second;
		VarWidget * w = Gtk::manage (new VarWidget ((*iter).first, (*iter).second));
		ifoptVBox.pack_start (*w);
	}
	ifoptFrame.set_sensitive (true);

	typeoptVBox.children().clear();
	for (std::map <string, VarInstance>::const_iterator iter = iface.type_options.begin(); iter != iface.type_options.end(); iter++)
	{
		string optname = (*iter).first;
		VarInstance inst = (*iter).second;
		VarWidget * w = Gtk::manage (new VarWidget ((*iter).first, (*iter).second, false));
		typeoptVBox.pack_start (*w);
	}
	typeoptFrame.set_sensitive (true);

	defoptVBox.children().clear();
	for (std::map <string, VarInstance>::const_iterator iter = iface.default_options.begin(); iter != iface.default_options.end(); iter++)
	{
		string optname = (*iter).first;
		VarInstance inst = (*iter).second;
		VarWidget * w = Gtk::manage (new VarWidget ((*iter).first, (*iter).second, false));
		defoptVBox.pack_start (*w);
	}
	defoptFrame.set_sensitive (true);

	qosView.setRoot(iface.qosRoot);
	qosView.expand_all();
	show_all_children();
}

VarWidget::VarWidget () :
//HButtonBox (Gtk::BUTTONBOX_SPREAD),
VarInstance()
{
}

void VarWidget::set (const VarInstance i)
{
	varType = i.varType;
	varLabel = i.varLabel;
	varDescription = i.varDescription;
	varLevel = i.varLevel;
	enumVal = i.enumVal;
	strValue = i.strValue;
	boolValue = i.boolValue;
	intValue = i.intValue;
}

void VarWidget::set (const string s)
{
	varName = s;
}

void VarWidget::repack ()
{
	children().clear();
	using namespace Gtk;
	Label * label;
	switch (varType)
	{
		case VT_STRING:
		{
			label = manage (new Label);
			if (varLabel != "")
				label->set_label (varLabel);
			else
				label->set_label (varName);
			pack_start (*label);
			Entry * entry = manage (new Entry);
			entry->set_text (strValue);
			entry->set_sensitive (isImported);
			pack_start (*entry);
			break;
		}
		case VT_INT:
		{
			label = manage (new Label);
			if (varLabel != "")
				label->set_label (varLabel);
			else
				label->set_label (varName);
			pack_start (*label);
			SpinButton * spin = manage (new SpinButton);
			spin->set_digits (0);
			spin->set_increments (1, 10);
			spin->set_range (-9999, 9999);
			spin->set_value (intValue);
			spin->set_sensitive (isImported);
			pack_start (*spin);
			break;
		}
		case VT_BOOL:
		{
			CheckButton * toggle = manage (new CheckButton);
			if (varLabel != "")
				toggle->set_label (varLabel);
			else
				toggle->set_label (varName);
			toggle->set_active (boolValue);
			toggle->set_mode (true);
			toggle->set_sensitive (isImported);
			pack_start (*toggle);
			break;
		}
		case VT_ENUM:
		{
			label = manage (new Label);
			if (varLabel != "")
				label->set_label (varLabel);
			else
				label->set_label (varName);
			pack_start (*label);
			ComboBoxText * menu = manage (new ComboBoxText);
			list <string> tmplist;
			for (list <string>::const_iterator i = enumVal.begin(); i != enumVal.end(); i++)
				menu->append_text (*i);
			menu->set_active_text (strValue);
			menu->set_sensitive (isImported);
			pack_start (*menu);
			break;
		}
		default:
			// print warning
			break;
	}
	Button * button = manage (new Button);
	button->set_sensitive (false);
	if (isImported)
		button->set_label ("undefine");
	else
		button->set_label ("define");
	pack_start (*button);
	show_all_children();
}

void VarWidget::set (const string varName, const VarInstance i)
{
	set (varName);
	set (i);
	repack();
}

VarWidget::VarWidget (const string varName, const VarInstance i, const bool isImported)
{
	VarWidget();
	this->isImported = isImported;
	set (varName, i);
}
