#include "MainWin.hxx"
#include "Domain.hxx"
#include <iostream>

MainWin::MainWin() :
a (Gtk::ALIGN_CENTER, Gtk::ALIGN_CENTER, 1, 1)
{
	set_title ("/etc/net configurator");
	set_default_size (600, 400);
	add (vBox);


	actionGroup = Gtk::ActionGroup::create();
	actionGroup->add
	(
		Gtk::Action::create ("LoadDefault", Gtk::Stock::NEW, "_Load", "Load default configuration"),
		sigc::mem_fun(*this, &MainWin::on_load)
	);
	actionGroup->add (Gtk::Action::create ("Level1", "_User"), sigc::mem_fun(*this, &MainWin::on_level1));
	actionGroup->add (Gtk::Action::create ("Level2", "_System administrator"), sigc::mem_fun(*this, &MainWin::on_level2));
	actionGroup->add (Gtk::Action::create ("Level3", "_Network administrator"), sigc::mem_fun(*this, &MainWin::on_level3));
	actionGroup->add (Gtk::Action::create ("FileMenu", "File"));
	actionGroup->add (Gtk::Action::create ("LevelMenu", "Level"));
	actionGroup->add (Gtk::Action::create("FileQuit", Gtk::Stock::QUIT), sigc::mem_fun(*this, &MainWin::on_quit));
	uiManager = Gtk::UIManager::create();
	uiManager->insert_action_group(actionGroup);
	add_accel_group (uiManager->get_accel_group());

	try
	{
		Glib::ustring ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='LoadDefault'/>"
"      <menuitem action='FileQuit'/>"
"    </menu>"
"    <menu action='LevelMenu'>"
"      <menuitem action='Level1'/>"
"      <menuitem action='Level2'/>"
"      <menuitem action='Level3'/>"
"    </menu>"
"  </menubar>"
"  <toolbar  name='ToolBar'>"
"    <toolitem action='LoadDefault'/>"
"    <toolitem action='FileQuit'/>"
"  </toolbar>"
"</ui>";
		uiManager->add_ui_from_string(ui_info);
	}
	catch (const Glib::Error& ex)
	{
		cerr << "building menus failed: " <<  ex.what();
	}
  Gtk::Widget* pMenubar = uiManager->get_widget("/MenuBar");
  if(pMenubar)
    vBox.pack_start(*pMenubar, Gtk::PACK_SHRINK);

  Gtk::Widget* pToolbar = uiManager->get_widget("/ToolBar");
  if(pToolbar)
    vBox.pack_start(*pToolbar, Gtk::PACK_SHRINK);
	hPane.add1 (leftScroll);
	hPane.add2 (a);
	a.add (rightScroll);
	leftScroll.set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	rightScroll.set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	leftScroll.add (browseView);
	rightScroll.add (ifaceView);
	vBox.pack_start (hPane);
	show_all_children();
	Glib::RefPtr<Gtk::TreeSelection> selection;
	selection = browseView.get_selection();
	selection->signal_changed().connect (sigc::mem_fun (*this, &MainWin::on_selection_changed));
};

void MainWin::on_load ()
{
	browseView.clear();
	Domain domain;
	domain.scan();
	browseView.add (domain);
	browseView.expand_all();
}

void MainWin::on_quit()
{
	hide();
}

void MainWin::on_level1()
{
	printf ("Switching to level: User\n");
}

void MainWin::on_level2()
{
	printf ("Switching to level: System administrator\n");
}

void MainWin::on_level3()
{
	printf ("Switching to level: Network Administrator\n");
}

void MainWin::on_selection_changed()
{
	ifaceView.set (browseView.getSelectedIface());
}
