#include "QoS.hxx"
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#undef DEBUG

const string typenamebycode[] =
{
	"NULL",
	"qdisc",
	"class",
	"filter"
};

QoSObject::QoSObject()
{
	clear();
}

void QoSObject::clear()
{
	type = QOS_NULL;
	children.clear();
	dev = id = parent = params = myExtra = parentExtra = nodeIdPrefix = "";
}

const string QoSObject::strType()
{
	return typenamebycode[type];
}

const string QoSObject::getID()
{
	return id;
}

bool QoSObject::test_f (const string filename)
{
#ifdef DEBUG
	printf ("DEBUG: in QoSObject::test_f (%s)\n", filename.c_str());
#endif
	struct stat statbuf;
	if (lstat (filename.c_str(), &statbuf)) // it's okay
		return false;
	return S_ISREG (statbuf.st_mode);
}

// this method works like /etc/net/scripts/config-ipv4:process_qos_dir
bool QoSObject::buildFromDir
(
	const string where,
	const string id, // don't calculate basename here, we can do it in the caller
	const string parent,
	string last_qdisc_handle,
	const string parentExtra
)
{
#ifdef DEBUG
	printf ("DEBUG: in QoSObject::buildFromDir (%s)\n", where.c_str());
#endif
	// find if we are class or qdisc
	if (test_f (where + "/class"))
	{
		type = QOS_CLASS;
		nodeIdPrefix = "classid";
		this->id = last_qdisc_handle + ":" + id;
		readParams (where + "/class");
		if (test_f (where + "/filter"))
			readFilters (where + "/filter");
	}
	else if (test_f (where + "/qdisc"))
	{
		type = QOS_QDISC;
		nodeIdPrefix = "handle";
		this->id = id + ":";
		readParams (where + "/qdisc");
		last_qdisc_handle = id;
	}
	else
	{
		type = QOS_NULL;
		return false;
	}
	this->parent = parent;
	this->parentExtra = parentExtra;
	if (test_f (where + "/extra"))
		readExtra (where + "/extra");
#ifdef DEBUG
	printf ("DEBUG: iterating over children\n");
#endif
	// iterate over children, if any
	// FIXME: call setDev
	DIR * target = opendir (where.c_str());
	if (!target)
	{
		perror ("QoSObject::buildFromDir");
		return false;
	}
	struct dirent * dirEntry = NULL;
	while ((dirEntry = readdir (target)))
	{
#ifdef DEBUG
		printf ("DEBUG: iterating over child %s\n", dirEntry->d_name);
#endif
		struct stat statbuf;
		if (lstat ((where + "/" + dirEntry->d_name).c_str(), &statbuf))
		{
			perror ("lstat");
			continue;
		}
		if (!S_ISDIR (statbuf.st_mode))
			continue;
		if (dirEntry->d_name[0] == '.')
			continue;
		if (string (dirEntry->d_name) == "CVS")
			continue;
		// do the magic!
		QoSObject child;
		child.buildFromDir (where + "/" + dirEntry->d_name, dirEntry->d_name, id, last_qdisc_handle, myExtra);
		children.push_back (child);
	}
	closedir (target);
	// FIXME
	return false;
}

bool QoSObject::install()
{
/*
$TC $NODETYPE add dev $NAME $PARENTNODE $NODEIDPREFIX $NODEID $EXTRAATTRS $NODEATTRS
NODETYPE = typenamebycode[type]
NAME = dev
PARENTNODE = parent
NODEIDPREFIX = nodeIdPrefix
NODEID = id
EXTRAATTRS = parentExtra
NODEATTRS = params
*/
	
	return false;
}

void QoSObject::readParams (const string filename)
{
	params = "";
}

void QoSObject::readExtra (const string filename)
{
	myExtra = "";
}

void QoSObject::readFilters (const string filename)
{
}

void QoSObject::setDev (const string dev)
{
	this->dev = dev;
	for (list<QoSObject>::iterator i = children.begin(); i != children.end(); i++)
		(*i).setDev (dev);
}

void QoSObject::dump (const unsigned indent)
{
	for (unsigned col = 0; col < indent; col++)
		printf (" ");
	printf ("%s %s\n", id.c_str(), strType().c_str());
	for (list<QoSObject>::iterator i = children.begin(); i != children.end(); i++)
		(*i).dump (indent + 1);
}

QoSView::QoSView()
{
	qosModel = Gtk::TreeStore::create (qosColumns);
	set_model (qosModel);
	append_column ("ID", qosColumns.id);
	append_column ("type", qosColumns.type);
	set_size_request (200);
}

void QoSView::setRoot (QoSObject root)
{
#ifdef DEBUG
	printf ("DEBUG: in QoSView::setRoot()");
	root.dump();
#endif
	this->root = root;
	rebuild();
}

void QoSView::rebuild()
{
	qosModel->clear();
	processQoSObjectChildren (root, qosModel->append());
	columns_autosize();
}

void QoSView::processQoSObjectChildren (QoSObject root, Gtk::TreeIter row)
{
	(*row)[qosColumns.id] = root.getID();
	(*row)[qosColumns.type] = root.strType();
	if (root.children.size() == 0)
		return;
	for (list <QoSObject>::iterator child = root.children.begin(); child != root.children.end(); child++)
		processQoSObjectChildren (*child, qosModel->append (row->children()));
}

void QoSView::clear()
{
	root.clear();
	rebuild();
}
