#include "VarInfo.hxx"

void VarInfo::set (const VarType varType, const string varLabel, const string varDescription, const unsigned varLevel)
{
	this->varType = varType;
	this->varLabel = varLabel;
	this->varDescription = varDescription;
	this->varLevel = varLevel;
	enumVal.clear();
	if (varType == VT_BOOL)
	{
		enumVal.push_back ("on");
		enumVal.push_back ("off");
	}
}

VarInfo::VarInfo (const VarType varType, const string varLabel, const string varDescription, const unsigned varLevel)
{
	set (varType, varLabel, varDescription, varLevel);
}

void VarInfo::addEnumVal (const string val)
{
	enumVal.push_back (val);
}

VarType VarInfo::getType()
{
	return varType;
}

bool VarInfoMap::infoExists (const string varName)
{
	return infoMap.find (varName) != infoMap.end();
}

VarInfo VarInfoMap::getInfo (const string varName)
{
	VarInfo vi;
	map <string, VarInfo>::const_iterator i = infoMap.find (varName);
	if (i != infoMap.end())
		vi = (*i).second;
	return vi;
}

void VarInfoMap::add (const string varName, const VarInfo varInfo)
{
	infoMap[varName] = varInfo;
}

void VarInfoMap::loadDefaults (VarInfoMap & m)
{
	VarInfo vi;
	// first level
	vi.set (VT_BOOL, "Disable", "Set this to yes to disable entire interface", 1);
	m.add ("DISABLED", vi);

	vi.set (VT_ENUM, "Type", "Interface type. Select one from several possible interface types.", 1);
	vi.addEnumVal ("lo");
	vi.addEnumVal ("dummy");
	vi.addEnumVal ("eth");
	vi.addEnumVal ("plip");
	vi.addEnumVal ("usb");
	vi.addEnumVal ("dvb");
	vi.addEnumVal ("vlan");
	vi.addEnumVal ("bond");
	vi.addEnumVal ("bri");
	vi.addEnumVal ("teql");
	vi.addEnumVal ("iptun");
	vi.addEnumVal ("ipsectun");
	vi.addEnumVal ("ppp");
	m.add ("TYPE", vi);
	
	vi.set (VT_BOOL, "Process on boot", "Disable this setting to skip interface init during system boot. You will be able to bring it up manuall later with ifup.", 1);
	m.add ("ONBOOT", vi);

	vi.set (VT_ENUM, "Boot protocol", "Boot protocol determines the way interface address(es) and route(s) are assigned.", 1);
	vi.addEnumVal ("static");
	vi.addEnumVal ("dhcp");
	vi.addEnumVal ("ipv4ll");
	vi.addEnumVal ("dhcp-static");
	vi.addEnumVal ("dhcp-ipv4ll");
	vi.addEnumVal ("dhcp-ipv4ll-static");
	m.add ("BOOTPROTO", vi);

	// second level, common options

	vi.set (VT_BOOL, "Use PCMCIA", "If you are setting up 16-bit PCMCIA interface, enable this option. Such interfaces will be skipped during boot, but after card insertion cardmgr (PCMCIA daemon) will run appropriate script to configure the interface. Make sure your pcmcia-cs package is patched to work with /etc/net.", 2);
	m.add ("USE_PCMCIA", vi);

	vi.set (VT_BOOL, "Use HOTPLUG", "This options is similar to USE_PCMCIA, but is intended to use with 32-bit removable devices, which are handled by hotplug package. hotplug requires a patch too to work with /etc/net.", 2);
	m.add ("USE_HOTPLUG", vi);

	vi.set (VT_STRING, "Kernel module", "Fill in this option, if you wish /etc/net to handle kernel module insertion and removal without dependency on /etc/modules.conf, pcmcia-cs or hotplug.", 2);
	m.add ("MODULE", vi);

	vi.set (VT_BOOL, "Keep down", "Perform all interface configuration up to setting it UP, but exit instead of setting UP", 2);
	m.add ("KEEP_DOWN", vi);

	vi.set (VT_BOOL, "Don't flush", "Don't flush interface addresses when setting it down.", 2);
	m.add ("DONT_FLUSH", vi);

	// second level, type-specific tools locations

	vi.set (VT_STRING, "brctl path", "This option allows you to override default path to brctl, which is used to setup ethernet bridge (bri) interfaces.", 2);
	m.add ("BRCTL", vi);

	vi.set (VT_STRING, "pppd(8) path", "This option allows you to override default path to pppd, which is used to setup PPP (ppp) interfaces.", 2);
	m.add ("PPPD", vi);

	vi.set (VT_STRING, "chat(8) path", "This option allows you to override default path to chat, which is used to setup PPP (ppp) interfaces.", 2);
	m.add ("CHAT", vi);

	vi.set (VT_STRING, "ifenslave path", "This option allows you to override default path to ifenslave, which is used to setup bonding (bond) interfaces.", 2);
	m.add ("IFENSLAVE", vi);

	vi.set (VT_STRING, "ipsecadm(8) path", "This option allows you to override default path to ipsecadm, which is used to setup ipsec_tunnel (ipsectun) interfaces.", 2);
	m.add ("IPSECADM", vi);

	vi.set (VT_STRING, "iwconfig(8) path", "This option allows you to override default path to iwconfig, which is used to setup WEP wireless extensions to ethernet (eth) interfaces.", 2);
	m.add ("IWCONFIG", vi);

	vi.set (VT_STRING, "wpa_supplicant path", "This option allows you to override default path to wpa_supplicant, which is used to setup WPA wireless extensions to ethernet (eth) interfaces.", 2);
	m.add ("WPA_SUPPLICANT", vi);

	vi.set (VT_STRING, "pentanett path", "This option allows you to override default path to pentanett, which is used to setup Pent@NET (pentanet) interfaces.", 2);
	m.add ("PENTANETT", vi);

	vi.set (VT_STRING, "vconfig path", "This option allows you to override default path to vconfig, which is used to setup VLAN (vlan) interfaces.", 2);
	m.add ("VCONFIG", vi);

	// second level, type-specific options

	vi.set (VT_INT, "MAC address wait time", "1 unit = 0.1 sec. Several ethernet devices appear with empty MAC address and aqquire a valid one anly after some time period. If you use 'mac' descriptor in iftab(5), this can become a problem. Specify maximum amount of time that can be spent waiting for a valid MAC address here.", 2);
	m.add ("MACADDR_WAITTIME", vi);

	vi.set (VT_ENUM, "WPA driver", "[unfinished] WPA driver as defined by wpa_supplicant -D option.", 2);
	vi.addEnumVal ("wext");
	vi.addEnumVal ("prism54");
	m.add ("WPA_DRIVER", vi);

	vi.set (VT_BOOL, "Detect link", "Link detection is useful for ethernet devices which use non-static BOOTPROTO. Enabled value will prevent dynamic configuration attempt if no link is detected. This options is designed to prevent timeouts during system boot.", 2);
	m.add ("LINKDETECT", vi);

	vi.set (VT_STRING, "DHCP hostname", "DHCP client behaves differently depending on the value of this option. 'localhost' causes it to compute a valid hostname and to send it to DHCP server to identify client (-h). 'AUTO' causes setting hostname from DHCP server response (-H). Any other value is sent as a hostname to DHCP server (-h).", 2);
	m.add ("DHCP_HOSTNAME", vi);

	vi.set (VT_ENUM, "Use ifplugd", "If you have ifplugd installed, it can be automatically launched by /etc/net. Your ethernet interface will be configured accordingly to current link status. If you enable this option, make sure that your interface reports link status correctly. Alternatively, use 'auto' to enable this feature only if it will work with your system.", 2);
	vi.addEnumVal ("yes");
	vi.addEnumVal ("no");
	vi.addEnumVal ("auto");
	m.add ("USE_IFPLUGD", vi);

	vi.set (VT_STRING, "ifplugd extra args", "Here you can set additional arguments ifplugd should receive upon start, e.g. --up-delay or --down-delay.", 2);
	m.add ("IFPLUGD_EXTRA_ARGS", vi);

	vi.set (VT_BOOL, "persistent ifplugd", "This is an experimental feature. If enabled, each interface's ifplugd process will not be shutdown on ifdown, but suspended and resumed on ifup.", 2);
	m.add ("PERSISTENT_IFPLUGD", vi);

	vi.set (VT_ENUM, "Tunnel type", "Each tunnel must belong to some tunnel type. If not set, default value will be used.", 2);
	vi.addEnumVal ("ipip");
	vi.addEnumVal ("gre");
	vi.addEnumVal ("sit");
	m.add ("TUNTYPE", vi);

	vi.set (VT_STRING, "Tunnel local address", "IPv4 source address for tunneled packets. Can be 'any' for iptun, but not for ipsectun.", 2);
	m.add ("TUNLOCAL", vi);

	vi.set (VT_STRING, "Tunnel remote address", "IPv4 remote address for tunneled packets. Can be 'any' for iptun, but not for ipsectun.", 2);
	m.add ("TUNREMOTE", vi);

	vi.set (VT_STRING, "Tunnel additional options", "Can be used to configure tunnel packets TTL. Look up ip(8) manpage, 'tunnel configuration' for additional tunnel options.", 2);
	m.add ("TUNOPTIONS", vi);

	vi.set (VT_STRING, "IPSec cipher", "This option defines algorythm used to encrypt data. Value can be {3des,aes,blowfish,blowfish_old,cast5,des,dfc,idea,mars,null,nush,rc5,rc6,serpent,twofish,wicker}-{ecb,cbc,cfb,ctr,rtc}, but this is kernel-dependant. Either cipher or digest must be configured for ipsectun.", 2);
	m.add ("CIPHER", vi);

	vi.set (VT_ENUM, "IPSec digest", "This option defines algorythm used to sign data. Possible values are kernel-dependant. Either cipher or digest must be configured for ipsectun.", 2);
	vi.addEnumVal ("md5");
	vi.addEnumVal ("ripemd160");
	vi.addEnumVal ("sha1");
	vi.addEnumVal ("sha256");
	vi.addEnumVal ("sha384");
	vi.addEnumVal ("sha512");
	m.add ("DIGEST", vi);

	vi.set (VT_INT, "VLAN ID", "Required for a VLAN interface. Value is integer between 1 (not recommended) and 4095.", 2);
	m.add ("VID", vi);

	vi.set (VT_ENUM, "Bonding mode", "This mode defines the way bonding interface will use its slave interfaces. See modinfo bond for more info.", 2);
	vi.addEnumVal ("0");
	vi.addEnumVal ("1");
	vi.addEnumVal ("2");
	vi.addEnumVal ("3");
	vi.addEnumVal ("4");
	m.add ("BONDMODE", vi);

	vi.set (VT_STRING, "Bonding options", "Specify additional kernel module options here.", 2);
	m.add ("BONDOPTIONS", vi);

	vi.set (VT_ENUM, "PPP interface type", "If you want to save time configuring other than dialup PPP link, you can choose a different type here. pptp type will require PPTP_SERVER option, while pppoe one will require HOST.", 2);
	vi.addEnumVal ("dialup");
	vi.addEnumVal ("pptp");
	vi.addEnumVal ("pppoe");
	m.add ("PPPTYPE", vi);

	vi.set (VT_INT, "PPP timeout", "PPP link timeout", 2);
	m.add ("PPPTIMEOUT", vi);

	vi.set (VT_BOOL, "Restore default route", "/etc/net can save default route before bringing PPP link up and restore it after disconnect.", 2);
	m.add ("RESTORE_DEFAULTROUTE", vi);

	vi.set (VT_STRING, "Additional pppd options", "man pppd(8)", 2);
	m.add ("PPPOPTIONS", vi);

	vi.set (VT_STRING, "Additional chat options", "man chat(8)", 2);
	m.add ("CHATOPTIONS", vi);

	vi.set (VT_STRING, "Host interface(s)", "Space-separated list of interface(s) which form VLAN/bonding/bridge/teql interface", 2);
	m.add ("HOST", vi);

	vi.set (VT_ENUM, "DVB interface type", "One of: pentanet (Pent@NET), ss223 (SkyStar-2 rev. 2.3), ss226 (SkyStar-2 rev. 2.6).", 2);
	vi.addEnumVal ("pentanet");
	vi.addEnumVal ("ss223");
	vi.addEnumVal ("ss226");
	m.add ("DVBTYPE", vi);

	// second level, type-specific file names

	vi.set (VT_STRING, "Cipher key location", "Cipher requires a key to encrypt data. Key length depends on the cipher. If you have configured cipher for the interface, store cipher key in a binary file in current interface directory under /etc/net/ifaces and specify file name here.", 2);
	m.add ("CIPHERFILE", vi);

	vi.set (VT_STRING, "Digest key location", "Digest requires a key to sign data. Key length depends on the digest. If you have configured digest for the interface, store digest key in a binary file in current interface directory under /etc/net/ifaces and specify file name here.", 2);
	m.add ("DIGESTFILE", vi);

	vi.set (VT_STRING, "szap path", "Full path to szap utility, which is used to setup SkyStar-2 DVB cards.", 2);
	m.add ("SZAP", vi);

	vi.set (VT_STRING, "szap arguments", "szap arguments", 2);
	m.add ("SZAP_ARGS", vi);

	vi.set (VT_STRING, "dvbnet path", "Full path to dvbnet utility, which is used to setup SkyStar-2 DVB cards.", 2);
	m.add ("DVBNET", vi);

	vi.set (VT_STRING, "pentanet.conf location", "Pent@NET cards require additional configuration, which is performed by pentanett tool, which requires a configuration file. Store the file in current interface directory under /etc/net/ifaces and specify its name here (or use a default file name pentanet.conf and leave this option unset).", 2);
	m.add ("PENTANET_CONF", vi);

	vi.set (VT_STRING, "channels.conf location", "SkyStar cards require additional configuration, which is performed by szap tool, which requires channels.conf file.", 2);
	m.add ("CHANNELS_CONF", vi);

	// third level

	vi.set (VT_BOOL, "Configure IPv4", "This option lets you explicitly set whether to configure IPv4 protocol stack for this interface or not.", 3);
	m.add ("CONFIG_IPV4", vi);

	vi.set (VT_BOOL, "Configure IPv6", "This option lets you explicitly set whether to configure IPv6 protocol stack for this interface or not.", 3);
	m.add ("CONFIG_IPV6", vi);

	vi.set (VT_BOOL, "Configure IPX", "This option lets you explicitly set whether to configure IPX protocol stack for this interface or not.", 3);
	m.add ("CONFIG_IPX", vi);

	vi.set (VT_BOOL, "Configure QoS", "Each interface can have a QoS configuration tree. If you want to override default setting and enable/disable QoS configuration processing, this is the right option.", 3);
	m.add ("CONFIG_QOS", vi);

	vi.set (VT_BOOL, "Configure wireless", "Wireless extensions are by default enabled for ethernet interfaces. Change this option to override this default.", 3);
	m.add ("CONFIG_WIRELESS", vi);

	vi.set (VT_BOOL, "Configure firewall", "This option controls whether firewall configuration is performed.", 3);
	m.add ("CONFIG_FW", vi);

	vi.set (VT_STRING, "Required interface(s)", "Space-separated list of interface(s) required to bring current interface up (interfaces that should be up before the current interface configuration starts). See manual for more info.", 3);
	m.add ("REQUIRES", vi);

	vi.set (VT_BOOL, "ifup children", "Bring children up after current interface is up.", 3);
	m.add ("IFUP_CHILDREN", vi);

	vi.set (VT_BOOL, "ifup parents", "Bring parents up before current interface is up.", 3);
	m.add ("IFUP_PARENTS", vi);

	vi.set (VT_BOOL, "ifdown children", "Bring children down before current interface is down.", 3);
	m.add ("IFDOWN_CHILDREN", vi);

	vi.set (VT_BOOL, "ifdown parents", "Bring parents down after current interface is down.", 3);
	m.add ("IFDOWN_PARENTS", vi);

	vi.set (VT_BOOL, "automatic broadcast address", "Add missing broadcast address to ipv4address contents.", 3);
	m.add ("AUTO_BROADCAST", vi);
}
