#include "VarMap.hxx"
#define MAXLINELEN 1024
#include <stdio.h>
#include <strings.h>
#include <ctype.h>

void VarMap::load (const string filename, VarInfoMap info)
{
	varMap.clear();
	FILE *inf = NULL;
	if (!(inf = fopen (filename.c_str(), "r"))) return;
	char line[MAXLINELEN];
	bzero (line, MAXLINELEN);
	while (!feof (inf))
	{
		fgets (line, MAXLINELEN, inf);
		if (line[0] == '#' || line[0] == '\n') continue;
		unsigned cpos = 0;
		// skip spaces
		while (line[cpos] && isspace (line[cpos])) cpos++;
		unsigned pos = cpos, len = 0;
		// pass varname
		while (line[cpos] && (isalnum (line[cpos]) || line[cpos] == '_')) { cpos++; len++; }
		string key (line, pos, len);
		// skip spaces/=
		while (line[cpos] && (isspace (line[cpos]) || line[cpos] == '=')) cpos++;
		// now cpos points at either the first value char or line end
		pos = cpos;
		len = 0;
		while (line[cpos] && line[cpos] != '\n') { cpos++; len++; }
		string value (line, pos, len);
		VarInstance inst (info.getInfo (key));
		inst.setStrValue (value);
		varMap[key] = inst;
	}
	fclose (inf);
}

bool VarMap::exists (const string key)
{
	return (varMap.find (key) != varMap.end());
}

string & VarMap::operator[] (const string key)
{
	return varMap[key].strValue;
}

void VarMap::dump()
{
	for (map <string, VarInstance>::const_iterator i = varMap.begin(); i != varMap.end(); i++)
	{
		pair<string, VarInstance> pair = *i;
		printf ("'%s' == '%s'\n", pair.first.c_str(), pair.second.strValue.c_str());
	}
}

void VarMap::subtract (const VarMap & v)
{
	for (map <string, VarInstance>::const_iterator i = v.varMap.begin(); i != v.varMap.end(); i++)
		if (varMap.find ((*i).first) != varMap.end())
			varMap.erase ((*i).first);
}

map <string, VarInstance>::const_iterator VarMap::begin()
{
	return varMap.begin();
}

map <string, VarInstance>::const_iterator VarMap::end()
{
	return varMap.end();
}

const string VarInstance::getStrValue()
{
	return strValue;
}

const VarType VarInstance::getType()
{
	return varType;
}

bool VarInstance::is_yes (const string s)
{
	// 1
	if (s.size() == 1 && s[0] == '1') return true;
	// on
	if (s.size() == 2 && (s[0] == 'o' || s[0] == 'O') && (s[1] == 'n' || s[1] == 'N')) return true;
	// yes
	if (s.size() == 3 && (s[0] == 'y' || s[0] == 'Y') && (s[1] == 'e' || s[1] == 'E') && (s[2] == 's' || s[2] == 'S')) return true;
	// true
	if (s.size() == 4 && (s[0] == 't' || s[0] == 'T') && (s[1] == 'r' || s[1] == 'R') && (s[2] == 'u' || s[2] == 'U') && (s[3] == 'e' || s[3] == 'E')) return true;
	return false;
}

void VarInstance::setStrValue (const string value)
{
	strValue = value;
	switch (varType)
	{
		case VT_INT:
			intValue = atoi (strValue.c_str());
			break;
		case VT_BOOL:
			if (is_yes (strValue))
				boolValue = true;
			else
				boolValue = false;
			break;
		case VT_ENUM:
			break;
		default: // do nothing
			break;
	}
}

VarInstance::VarInstance (const VarInfo & info)
{
	intValue = 0;
	boolValue = false;
	varType = info.varType;
	varLabel = info.varLabel;
	varDescription = info.varDescription;
	varLevel = info.varLevel;
	enumVal = info.enumVal;
}

VarInstance::VarInstance()
: VarInfo()
{
	intValue = 0;
	boolValue = false;
}
